//TT UI Builder
var TT_UI_Builder = function( tableBodyId, guiTexts ) { 
	
	this.builder.initialize( tableBodyId, guiTexts );
	
	this.getLabel = function( id ) {
		return( this.builder.textHandler.getCaption(id) );
	};
	
	this.assignCaption = function( elementId ) {
		
		var captionText = this.getLabel( elementId );
		if( !captionText ) {
			return;
		}

		var captionElement = $( "#" + elementId );
		if( !captionElement ) {
			return;
		}
		
		captionElement.html( captionText );
	};

	this.assignDescription = function( elementId ) {
		
		var descriptionText = this.getLabel( elementId );
		if( !descriptionText ) {
			return;
		}
		
		var descriptionElement =  $( "#" + elementId );
		if( !descriptionElement ) {
			return;
		}
		
		descriptionElement.html( descriptionText );
	};
	
	this.assignOkButton = function( buttonId ) {
		
		var okButton = $( "#" + buttonId );
		okButton.attr( "value", this.getLabel( buttonId ) );
		okButton.bind( "click", function( event ){
			saveChanges();
			closeDialog();
		});		
		okButton.button();
	};
	
	this.assignCancelButton = function( buttonId ) {

		var cancelButton = $( "#" + buttonId );
		cancelButton.attr( "value", this.getLabel( buttonId ) );
		cancelButton.bind( "click", function( event ){
			closeDialog();
		});
		cancelButton.button();
	};
	
	this.addSeperator = function() {
		this.builder.createSeperator();
	};

	this.addEntryDescription = function( id ) {
		this.builder.createEntryDescription( id );
	};
	
	this.addEditEntry = function( id ) {
		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createEditCell( id );
		this.builder.createEntry( idCell, valueCell );
	};

	this.addTextAreaEntry = function( id, rows, cols ) {
		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createTextAreaCell( id, rows, cols );
		this.builder.createEntry( idCell, valueCell );
	};
	
	this.addCompoBoxEntry = function( id, comboList ) {
		
		if( !comboList ) {
			return;
		}

		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createComboBoxCell( id, comboList );
		this.builder.createEntry( idCell, valueCell );
	};
	
	this.addFileSelectionEntry = function( id ) {
		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createFileSelectionCell( id );
		this.builder.createEntry( idCell, valueCell );
	};
	
	this.addCheckBoxEntry = function( id )  {
		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createCheckBoxCell( id );
		this.builder.createEntry( idCell, valueCell );
	};
	
	this.addCustomEntry = function( id, element ) {
		var idCell = this.builder.createIdCell( id );
		var valueCell = this.builder.createCustomCell( id, element );
		this.builder.createEntry( idCell, valueCell );
	};
};


TT_UI_Builder.prototype.builder = {

	tableBody : null,
	
	initialize : function( tableBodyId, guiTexts ) {
		this.initializeTable( tableBodyId );
		this.textHandler.initialize( guiTexts );
	},
	
	textHandler : {

		currentLabels : null,
		defaultLanguage : "en-us",
		currentLanguage : "en-us",
		
		initialize : function( labels ) {
			
			this.currentLanguage = getGuiLanguage();
			if( !this.currentLanguage ) {
				this.currentLanguage = this.defaultLanguage ;
			}
			
			this.currentLabels = labels;
		},

		getCaption : function( id ) {
			
			var labelObject = this.getLabelObject( id );
			if( !labelObject  ) {
				return( id );
			}

			return( this.getLocalizedLabel(labelObject) );			
		},

		getLabelObject : function( id ) {
				
			if( !this.currentLabels ) {
				return( null);
			}
			
			var labels = this.currentLabels; 
			if( id.indexOf( TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE + "." ) == 0 ) {
				labels = labels[TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE];
				id = id.substring( TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE.length + 1 );
			}

			if( !labels ) {
				return( null);
			}
			
			if( typeof( labels[id] ) != "undefined" ) {
				return( labels[id] );
			}

			return( null );
		},

		getLocalizedLabel : function( labelObject ) {
				
			if( !labelObject ) {
				return( null);
			}

			if( typeof( labelObject[this.currentLanguage] ) != "undefined" ) {
				return( labelObject[this.currentLanguage] );
			}

			if( typeof( labelObject[this.defaultLanguage] ) != "undefined" ) {
				return( labelObject[this.defaultLanguage] );
			}
			
			return( labelObject );
		}
	},
	
	initializeTable : function( id ) {

		this.tableBody = $( "#" + id );
		if( !this.tableBody ) {
			return( false );
		}
		
		this.tableBody.addClass( "ParameterTable" );
		
		var formatRow = this.createRow();
		formatRow.css( "display", "none" );
		
		var formatCell = $(document.createElement("td"));
		formatCell.html( "Name" );
		formatCell.addClass( "IdColCaption" );
		formatRow.append( formatCell );
		
		formatCell = $(document.createElement("td"));
		formatCell.html( "Value" );
		formatCell.addClass( "ValueColCaption" );
		formatRow.append( formatCell );

		this.tableBody.append( formatRow );
		
		return( true );
	},
	
	createRow : function() {
		var row = $(document.createElement("tr"));
		return( row );
	},

	createIdCell : function( id ) {
		var cell = $(document.createElement("td"));
		cell.addClass( "EntryId" );
		cell.html( this.textHandler.getCaption(id) );
		return( cell );
	},

	createValueCell : function( id ) {
		var cell = $( document.createElement("td") );
		cell.addClass( "EntryValue" );
		return( cell );
	},
	
	createEditCell : function( id ) {

		var cell = this.createValueCell();

		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );
		
		var editElement = $( document.createElement("input") );
		editElement.addClass( "EntryValueEdit" );
		editElement.attr( "id", id );
		editElement.attr( "value", ValueUtil.get( id ) );
		editElement.bind( "keyup", {"editId":id}, function(event) {
			var id = event.data.editId;
			var value = editElement.attr("value");
			ValueUtil.set( id, value );
		});

		cellDiv.append( editElement );
		
		return( cell );
	},

	createTextAreaCell : function( id, rows, cols ) {

		var cell = this.createValueCell();
		
		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );

		var editElement = $( document.createElement("textarea") );
		editElement.addClass( "EntryValueTextArea" );
		editElement.attr( "id", id );
		editElement.attr( "rows", rows );
		editElement.attr( "cols", cols );
		
		var value = ValueUtil.get( id );
		if( value ) {
			value = value.replace( /\<br\/\>/g, "\n\r" );
			editElement.attr( "value", value );
		}

		editElement.bind( "keyup", {"editId":id}, function( event ) {
			var id = event.data.editId;
			var targetObject = $( "#" + id );
			var value = targetObject.attr("value");
			if( value ) {
				value = value.replace( /\n\r?/g, "<br/>" );
				value = value.replace( /\r?/g, "" );
			}
			ValueUtil.set( id, value );
		});

		cellDiv.append( editElement );
		
		return( cell );
	},
	
	createComboBoxCell : function( id, comboList ) {

		var cell = this.createValueCell();

		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );

		var valueSelectElement = $(document.createElement("select"));
		valueSelectElement.addClass( "EntryValueComboBox" );
		valueSelectElement.attr( "id", id );
		
		for( var i=0; i<comboList.length; i++ ) {
			var value = comboList[i].value;
			var label = comboList[i].label;
			$("<option/>").val( value ).text( label ).appendTo( valueSelectElement );
		}
		
		valueSelectElement.attr( "value", ValueUtil.get(id) );
		valueSelectElement.bind( "change", {"editId":id}, function( event ) {
			var id = event.data.editId;
			var targetObject = $( "#" + id );
			var value = targetObject.val();
			ValueUtil.set( id, value );
		});
		
		cellDiv.append( valueSelectElement );
		
		return( cell );
	},
	
	createCheckBoxCell  : function( id ) {

		var cell = this.createValueCell();

		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );

		var checkBox = $(document.createElement("input"));
		checkBox.addClass( "EntryValueCheckBox" );
		checkBox.attr( "id", id );
		checkBox.attr( "type", "checkbox" ); 

		var value = ValueUtil.get(id);
		if( value ) {
			checkBox.attr( "checked", (value.toLowerCase() == "true") );
		}
		
		checkBox.bind( "mouseup", {"editId":id}, function( event ) {
			var id = event.data.editId;
			var targetObject = $( "#" + id );
			var value = targetObject.attr( "checked" );
			ValueUtil.set( id, value ? "false" : "true" );
		});
		
		cellDiv.append( checkBox );
		
		return( cell );
	},
	
	createFileSelectionCell : function( id ) {

		var cell = this.createValueCell();

		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );
		
		var editElement = $( document.createElement("input") );
		editElement.addClass( "EntryValueFileSelection" );
		editElement.attr( "id", id );
		editElement.attr( "value", ValueUtil.get( id ) );
		editElement.attr( "readonly", "true" );
		editElement.bind( "keyup", {"editId":id}, function( event ) {
			var id = event.data.editId;
			var editElement = $( "#" + id );
			if( editElement ) {
				var value = editElement.attr("value");
				if( !value ) {
					value = "";
				} 
				ValueUtil.set( id, value );
			}
		});
		
		cellDiv.append( editElement );

		var importButton = this.createFileSelectionButton( id )
		cellDiv.append( importButton );
		
		var trashButton = this.createTrashButton( id );
		cellDiv.append( trashButton );
		
		return( cell );
	},
	
	createFileSelectionButton : function( id ) {

		var importFileButton = $(document.createElement("span"));
		importFileButton.attr( "id", "importFile_" + id );
		importFileButton.addClass( "ui-state-default" );
		importFileButton.addClass( "ui-corner-all" );
		importFileButton.addClass( "EntryControlButton" );

		var importFileIcon = $(document.createElement("span"));
		importFileIcon.addClass( "ui-icon" );
		importFileIcon.addClass( "ui-icon-folder-open" );
		importFileButton.append( importFileIcon );
		
		importFileButton.bind( "mouseup", {"editId":id}, function( event ) {
			var id = event.data.editId; 
			var filePath = importFile(id);
			var editElement = $( "#" + id );
			if( editElement ) {
				editElement.attr( "value", filePath );
			}
			ValueUtil.set( id, filePath );
		});
		
		importFileButton.button();
		
		return( importFileButton );
	},

	createTrashButton : function( id ) {

		var trashButton = $(document.createElement("span"));
		trashButton.attr( "id", "trash_" + id );
		trashButton.addClass( "ui-state-default" );
		trashButton.addClass( "ui-corner-all" );
		trashButton.addClass( "EntryControlButton" );

		var trashIcon = $(document.createElement("span"));
		trashIcon.addClass( "ui-icon" );
		trashIcon.addClass( "ui-icon-trash" );
		trashButton.append( trashIcon );

		trashButton.bind( "mouseup", {"editId":id}, function( event ) {
			
			var id = event.data.editId;

			var editElement = $( "#" + id );
			if( editElement ) {
				editElement.attr( "value", "" );
			}
			ValueUtil.set( id, "" );
		});
		
		trashButton.button();
		
		return( trashButton );
	},

	createCustomCell : function( id, element ) {

		if( !element ) {
			return;
		}
		
		var cell = this.createValueCell();

		var cellDiv = $( document.createElement("div") );
		cellDiv.addClass( "EntryValueLayout" );
		cell.append( cellDiv );

		cellDiv.append( element );
		
		return( cell );

	},
	
	createEntry : function( idCell, valueCell, descriptionCell ) {

		if( !this.tableBody ) {
			return;
		}
		
		var entryRow = this.createRow();
		if( !idCell || !valueCell ) {
			return;
		}
			
		entryRow.append( idCell );
		entryRow.append( valueCell );
		
		this.tableBody.append( entryRow );

		if( descriptionCell ) {
			var infoRow = this.createRow();
			this.tableBody.append( infoRow );
		}			
	},

	createEntryDescription : function( id ) {

		if( !this.tableBody ) {
			return;
		}
		
		var description = this.textHandler.getCaption(id);
		if( !description || description == "" ) {
			return;
		}
		
		var row = this.createRow();

		var descriptionIdCellElement = $( document.createElement("td") );
		descriptionIdCellElement.addClass( "EntryDescriptionId" );
		row.append( descriptionIdCellElement );

		var descriptionCellElement = $( document.createElement("td") );
		descriptionCellElement.addClass( "EntryDescriptionValue" );
		descriptionCellElement.html( description );
		
		row.append( descriptionCellElement );
		
		this.tableBody.append( row );
	},
	
	createSeperator : function() {

		if( !this.tableBody ) {
			return;
		}
		
		var row = this.createRow();
		var seperatorCellElement = $( document.createElement("td") );
		seperatorCellElement.attr( "colSpan", 2 );
		seperatorCellElement.addClass( "Seperator" );

		row.append( seperatorCellElement );
		
		this.tableBody.append( row );
	}
};

var ValueUtil = {
	
	get : function( id ) {
		return( getParameterValue( id ) );
	},

	set : function( id, value ) {
		
		if( !value ) {
			value = "";
		}
		
		setParameterValue( id, value );		
	}
};

TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE = "com.tts";

//TT_UI_API functions mock
var setDialogSize = function( width, height ) {};
var getGuiLanguage = function() { return("de-de"); };
var getParameterList = function() { return("['url']"); };
var getParameterValue = function( id ) { return("true"); };
var setParameterValue = function( id, value ) {};
var importFile = function( id ) { return("file:///test.html") };
var saveChanges = function() {};
var closeDialog = function() {};
