define( ["jquery","jquery-ui"], function( jq ) {

	return function( context, metadata ) {
		
		// declare private variables
		var objShell = null,
			objAccordion = null;
			objShield = null,
			objAccordionWrapper = null,
			m_sTheme = "blue";
		
		// sets private variables to initial parameter values
		function initializeParameters() {

			m_sTheme = metadata.parameters.get( "theme", m_sTheme );
		}
		
		function createMarkup() {

			objShell = jq(context.getElement())
				.height( "auto" );		
		
			// if we are in athoring mode, create an object shield so we are able to select and drag element
			if( context.stage.isInAuthoringMode() ) {				
				var objShield = jq(document.createElement("div"));
				
				objShield.css( "position", "absolute" );
				objShield.css( "left", "0px" );
				objShield.css( "background-color", "#fff" );
				objShield.css( "opacity", "0.4" );
				objShield.css( "width", "100%" );
				objShield.css( "height", "100%" );
				objShield.css( "z-index", "9999" );
				
				objShell.append( objShield );
			}	
			
			objAccordionWrapper = jq(document.createElement("div"));
			
			for (var i = 1; i <= 3; i++) {
				var headline = metadata.parameters.get( "headline_" + i , "" );
				if( headline == "" ){
					headline = "Platzhalter Überschrift";
				}
				var content = metadata.parameters.get( "content_" + i , "" );
				var headlineObj = jq(document.createElement("h3"))
					.text( headline );
				var contentObj = jq(document.createElement("div"))
					.html('<p>' + content + '</p>');
				
				objAccordionWrapper.append( headlineObj );
				objAccordionWrapper.append( contentObj );
			}

			var objGreenStyleWrapper = jq(document.createElement("div"));
			switch( m_sTheme ){ 
				case "blue":
					objGreenStyleWrapper.addClass('blue');
					break;
				case "green":
					objGreenStyleWrapper.addClass('green');
					break;
				default:
					objGreenStyleWrapper.addClass('blue');
					break;
			}			
			objGreenStyleWrapper.append( objAccordionWrapper );
			
			objShell.append( objGreenStyleWrapper );			
		}	
		
		// the current TTLF version is not yet able to handle stylesheets within shared library. 
		// So this is a workaround that manually inserts the stylesheet within HTML page header node. 
		// In future versions this should be handled differently
		function insertJqueryUI_CSS( cssLocation ) {
			
			var snode = document.createElement('link');
			snode.setAttribute('rel','stylesheet');
			snode.setAttribute('href',cssLocation);
			document.getElementsByTagName('head')[0].appendChild(snode);
		}

		// revealing public API
		return {

			// declare public members

			onLoading : function() {
				
				context.page.event.bindOnEnter( this );	

				initializeParameters();
				
				// insert stylesheet within page header (workaround)
				switch( m_sTheme ){ 
					case "blue":
						insertJqueryUI_CSS( context.getPath() + "css/blue/custom-accordion.css" );
						break;
					case "green":
						insertJqueryUI_CSS( context.getPath() + "css/green/custom-accordion.css" );
						break;
					default:
						insertJqueryUI_CSS( context.getPath() + "css/blue/custom-accordion.css" );
						break;
				}
				
				createMarkup();
				
				objAccordionWrapper.accordion();
			},
			
			onAllComponentsReady : function () {
			
				objAccordionWrapper.accordion();
			},
			
			onEnterPage : function() {	
			}
		};		
	};
});

