define( ["jquery","amcharts"], function( jq ) {

	return function( context, metadata ) {
		
		// declare private variables
		var objShell = null,
			objChart = null,
			PLUGIN_1 = "PieDonut3D",
			PLUGIN_2 = "AnimatedBar",		
			chart_description = "",
			chart_width = 500,
			chart_height = 500,
			chart_scalingMode = "fixed",
			chart_plugin = "PieDonut3D";
		
		// sets private variables to initial parameter values
		function initializeParameters() {

			chart_description = metadata.parameters.get( "description", "some description ..." );
			chart_width = metadata.getWidth();
			chart_height = metadata.getHeight();
			chart_plugin = metadata.parameters.get( "chart_plugin" , PLUGIN_1 );

		}

		function createChartObject() {
			
				objShell = jq(context.getElement());
			
				// if we are in athoring mode, create an object shield so we are able to select and drag element
				if( context.stage.isInAuthoringMode() ) {				
					var objShield = jq(document.createElement("div"));
					
					objShield.css( "position", "absolute" );
					objShield.css( "left", "0px" );
					objShield.css( "background-color", "#ffffff" );
					objShield.css( "opacity", "0.4" );
					objShield.css( "width", "100%" );
					objShield.css( "height", "100%" );
					objShield.css( "z-index", "9999" );
					
					objShell.append( objShield );
				}
			
				// different html markup based on plugin used
				switch( chart_plugin ) {
					
					case PLUGIN_1:
						// create elements HTML markup
						objChart = jq(document.createElement("div"))					
							.attr("id", "chartdiv" )
							.attr('style', 'width: ' + chart_width + 'px; height: ' + chart_height + 'px;');
						break;
					
					case PLUGIN_2:
						// create elements HTML markup
						objChart = jq(document.createElement("div"))					
							.attr("id", "chartdiv" )
							.attr('style', 'width: ' + chart_width + 'px; height: ' + chart_height + 'px;');
						break;
						
					default:
						// create elements HTML markup
						objChart = jq(document.createElement("div"))					
							.attr("id", "chartdiv" )
							.attr('style', 'width: ' + chart_width + 'px; height: ' + chart_height + 'px;');
						break;
				}
				
				objShell.append( objChart
				);
		}	
		
		function animateChart(animationDuration) {
		
			switch( chart_plugin ) {
				
				case PLUGIN_1:	
					var chart;
		
					var chartData = [{
						country: "United States",
						visits: 9252
					}, {
						country: "China",
						visits: 1882
					}, {
						country: "Japan",
						visits: 1809
					}, {
						country: "Germany",
						visits: 1322
					}, {
						country: "United Kingdom",
						visits: 1122
					}, {
						country: "France",
						visits: 1114
					}, {
						country: "India",
						visits: 984
					}, {
						country: "Spain",
						visits: 711
					}];			
		
					// PIE CHART
					chart = new AmCharts.AmPieChart();
	
					// title of the chart
					chart.addTitle("Visitors countries", 16);
	
					chart.dataProvider = chartData;
					chart.titleField = "country";
					chart.valueField = "visits";
					chart.sequencedAnimation = true;
					chart.startEffect = "elastic";
					chart.innerRadius = "60%";
					chart.startDuration = parseInt(animationDuration);
					chart.labelRadius = 15;
	
					// the following two lines makes the chart 3D
					chart.depth3D = 20;
					chart.angle = 25;
	
					// WRITE                                 
					chart.write("chartdiv");	
					break;				

				case PLUGIN_2:
					var chart;
				
					var chartData = [{
						country: "Czech Republic",
						litres: 156.90,
						short: "CZ"
					}, {
						country: "Ireland",
						litres: 131.10,
						short: "IR"
					}, {
						country: "Germany",
						litres: 115.80,
						short: "DE"
					}, {
						country: "Australia",
						litres: 109.90,
						short: "AU"
					}, {
						country: "Austria",
						litres: 108.30,
						short: "AT"
					}, {
						country: "UK",
						litres: 99.00,
						short: "UK"
					}, {
						country: "Belgium",
						litres: 93.00,
						short: "BE"
					}, {
						country: "Test",
						litres: 38.00,
						short: "TE"
					}, {
						country: "Ireland",
						litres: 131.10,
						short: "IR"
					}, {
						country: "Germany",
						litres: 115.80,
						short: "DE"
					}, {
						country: "Australia",
						litres: 109.90,
						short: "AU"
					}, {
						country: "Austria",
						litres: 108.30,
						short: "AT"
					}, {
						country: "UK",
						litres: 99.00,
						short: "UK"
					}, {
						country: "Belgium",
						litres: 93.00,
						short: "BE"
					}, {
						country: "Test",
						litres: 38.00,
						short: "TE"
					}, {
						country: "Australia",
						litres: 109.90,
						short: "AU"
					}, {
						country: "Austria",
						litres: 108.30,
						short: "AT"
					}, {
						country: "UK",
						litres: 99.00,
						short: "UK"
					}, {
						country: "Belgium",
						litres: 93.00,
						short: "BE"
					}, {
						country: "Test",
						litres: 38.00,
						short: "TE"
					}, {
						country: "Australia",
						litres: 109.90,
						short: "AU"
					}, {
						country: "Austria",
						litres: 108.30,
						short: "AT"
					}, {
						country: "UK",
						litres: 99.00,
						short: "UK"
					}, {
						country: "Belgium",
						litres: 93.00,
						short: "BE"
					}, {
						country: "Test",
						litres: 38.00,
						short: "TE"
					}];
			
					// SERIAL CHART
					var chart = new AmCharts.AmSerialChart();
					chart.dataProvider = chartData;
					chart.categoryField = "country";
					chart.rotate = true;
					chart.color = "#FFFFFF";
			
					// this line makes the chart to show image in the background
					chart.backgroundImage = "";
			
					// sometimes we need to set margins manually
					// autoMargins should be set to false in order chart to use custom margin values 
					chart.autoMargins = true;
					chart.marginTop = 50;
					chart.marginLeft = 50;
					chart.marginRight = 30;
					chart.startDuration = parseInt(animationDuration);
			
					// AXES
					// category
					var categoryAxis = chart.categoryAxis;
					categoryAxis.gridAlpha = 0;
					categoryAxis.axisAlpha = 0;
					categoryAxis.labelsEnabled = true;
			
					// value
					var valueAxis = new AmCharts.ValueAxis();
					valueAxis.gridAlpha = 0;
					valueAxis.axisAlpha = 0;
					valueAxis.labelsEnabled = false;
					valueAxis.minimum = 0;
					chart.addValueAxis(valueAxis);
			
					// GRAPH
					var graph = new AmCharts.AmGraph();
					graph.balloonText = "[[category]]: [[value]]";
					graph.valueField = "litres"
					graph.type = "column";
					graph.lineAlpha = 0.5;
					graph.fillAlphas = 0.8;
					// you can pass any number of colors in array to create more fancy gradients
					graph.fillColors = ["#000000", "#FF6600"];
					graph.gradientOrientation = "horizontal";
					graph.labelPosition = "top";
					graph.labelText = "Test 1 [[category]]: [[value]] Litres";
					graph.balloonText = "Test 2 [[category]]: [[value]] Litres";
					chart.addGraph(graph);
			
					// LABEL
					chart.addLabel(50, 40, "Beer Consumption by country", "left", 15, "#FF0000", 0, 1, true);
			
					// WRITE
					chart.write("chartdiv");
					break;
			}
		}
		
		// revealing public API
		return {

			// declare public members

			onLoading : function() {
				
				initializeParameters();
				
				context.page.event.bindOnEnter( this );
				
				createChartObject();
				
				if (context.stage.isInAuthoringMode()) {
					animateChart(0);
					return;
				}
				
				animateChart(2);
			},
			
			onEnterPage : function() {		

				animateChart(2);
			}
		};
		
	};
});

