define( ["jquery","hoverpulse"], function( jq ) {

	return function( context, metadata ) {
		
		// declare private variables
		var objShell = null,
			objShield = null,
			objHoverpulse = null,
			m_nSpeed = 400,
			m_nSize = 40,
			m_sImage = context.getPath() + "default.png"
			m_nWidth = 58,
			m_nHeight = 58;
		
		// sets private variables to initial parameter values
		function initializeParameters( parameters ) {

			m_nSpeed = parameters.asNumber( "speed", "400" );
			m_nSize = parameters.asNumber( "size", "40" );		
			m_nWidth = parameters.asNumber( "wrapperWidth", m_nWidth );
			
			var Image = parameters.get( "picture", null );			
			if( Image ) {				
				m_sImage = context.stage.getSharedPath() + Image.fileName;
				m_nHeight = (m_nWidth / Image.width) * Image.height;
			} else {
				m_sImage = context.getPath() + "default.png";
				m_nHeight = (m_nWidth / 138) * 138;
			}	
			
			// substract zoom width from initial width, so the resolution is high enoigh
			m_nWidth =  m_nWidth;
			m_nHeight = m_nHeight;
		}
		
		function createMarkup() {

			objShell = jq(context.getElement())
				.width(m_nWidth + "px")
				.height(m_nHeight + "px");
		
			// if we are in athoring mode, create an object shield so we are able to select and drag element
			if( context.stage.isInAuthoringMode() ) {				
				objShield = jq(document.createElement("div"));
				
				objShield.css( "position", "absolute" );
				objShield.css( "left", "0px" );
				objShield.css( "background-color", "#fff" );
				objShield.css( "opacity", "0.4" );
				objShield.css( "width", m_nWidth + "px" );
				objShield.css( "height", m_nHeight + "px" );
				objShield.css( "z-index", "9999" );
				
				objShell.append( objShield );
			}	
			
			var objHoverpulseImg = jq(document.createElement("img"))
				.attr( 'src', m_sImage )
				.width( m_nWidth + 'px' )
				.height( m_nHeight + 'px' );
			
			objHoverpulse = jq(document.createElement("div"))
				.attr("id","thumbnails_" + metadata.getId())
				.width( m_nWidth  + 'px' )
				.height( m_nHeight + 'px' )
				.html( objHoverpulseImg );
			
			objShell.append( objHoverpulse );
		}
		
		// revealing public API
		return {

			// declare public members

			onLoading : function() {
				
				context.page.event.bindOnEnter( this );		
				
				initializeParameters( metadata.parameters );		
								
				createMarkup();
				
				if ( !context.stage.isInAuthoringMode()) {
					objHoverpulse.find(">:first-child").hoverpulse({
						size: m_nSize, // number of pixels to pulse element (in each direction)
						speed: m_nSpeed // speed of the animation 
					});
				}
			},
			
			onAllComponentsReady : function () {
				
				objHoverpulse.find(">:first-child").hoverpulse();
			},
			
			onEnterPage : function() {	
			}
		};		
	};
});

