define( ["jquery"], function( jq ) {

	return function( context, metadata ) {
		
		// declare private variables
		var objShell = null,
			objShield = null,
			objMagnifier = null,
			m_nOriginalWidth = 1000,
			m_nOriginalHeight = 568,
			m_nWidth = 500,
			m_nHeight = 284,
			m_nMagnifierWidth = 180,
			m_sImage = context.getPath() + "default.png"
		
		// sets private variables to initial parameter values
		function initializeParameters( parameters ) {
		
			m_nWidth = parameters.asNumber( "wrapperWidth", m_nWidth );
			
			var Image = parameters.get( "picture", null );
			if( Image ) {				
				m_sImage = context.stage.getSharedPath() + Image.fileName;
				m_nOriginalWidth = Image.width;
				m_nOriginalHeight = Image.height;
				m_nHeight = (m_nWidth / m_nOriginalWidth) * m_nOriginalHeight;
			} else {
				m_sImage = context.getPath() + "images/webpage.png";
			}
			
			m_nHeight = (m_nWidth / m_nOriginalWidth) * m_nOriginalHeight;
		}
		
		function createMarkup() {

			objShell = jq(context.getElement())
				.width(m_nWidth + "px")
				.height(m_nHeight + "px");
		
			// if we are in athoring mode, create an object shield so we are able to select and drag element
			if( context.stage.isInAuthoringMode() ) {				
				objShield = jq(document.createElement("div"));
				
				objShield.css( "position", "absolute" );
				objShield.css( "left", "0px" );
				objShield.css( "background-color", "#fff" );
				objShield.css( "opacity", "0.4" );
				objShield.css( "width", m_nWidth + "px" );
				objShield.css( "height", m_nHeight + "px" );
				objShield.css( "z-index", "9999" );
				
				objShell.append( objShield );
			}	

			objMagnifier = jq(document.createElement("div"))
				.html('<div class="magnifierWrapper"><div id="magnifierImageWrapper' + metadata.getId() + '" style="position: relative;"><img src="' + m_sImage + '" width="' + m_nWidth + '" height="' + m_nHeight + '" alt="Image" /><div id="retina' + metadata.getId() + '" style="background: url(\'' + m_sImage + '\') no-repeat center center white; border: 2px solid white; position: absolute; height: ' + m_nMagnifierWidth + 'px; width: ' + m_nMagnifierWidth + 'px; display: none; -moz-box-shadow:0 0 5px #777, 0 0 10px #aaa inset; -webkit-box-shadow:0 0 5px #777; box-shadow:0 0 5px #777, 0 0 10px #aaa inset; -moz-border-radius: ' + (m_nMagnifierWidth / 2) + 'px; -webkit-border-radius: ' + (m_nMagnifierWidth / 2) + 'px; border-radius: ' + (m_nMagnifierWidth / 2) + 'px; cursor: url(\'images/blank.cur\'), default;" class="retina"></div></div></div>');

			objShell.append( objMagnifier );
		}
		
		function initializeMagnifier(){
			var left	= 0,
				top		= 0,
				sizes	= { retina: { width: m_nMagnifierWidth, height: m_nMagnifierWidth }, imageWrapper:{ width: m_nWidth, height: m_nHeight } },
				imageWrapper	= $('#magnifierImageWrapper' + metadata.getId()),
				offset	= { left: imageWrapper.offset().left, top: imageWrapper.offset().top },
				retina	= $('#retina' + metadata.getId());

			if(navigator.userAgent.indexOf('Chrome')!=-1){
				/*	Applying a special chrome curosor,
					as it fails to render completely blank curosrs. */
					
				retina.css('cursor','url(\'images/blank_google_chrome.cur\'), default;');
			}
			
			imageWrapper.mousemove(function(e){

				left = (e.pageX-offset.left);
				top = (e.pageY-offset.top);

				if(retina.is(':not(:animated):hidden')){
					/* Fixes a bug where the retina div is not shown */
					imageWrapper.trigger('mouseenter');
				}

				if(left<0 || top<0 || left > sizes.imageWrapper.width || top > sizes.imageWrapper.height){
					/*	If we are out of the bondaries of the
						webpage screenshot, hide the retina div */

					if(!retina.is(':animated')){
						imageWrapper.trigger('mouseleave');
					}
					return false;
				}

				/*	Moving the retina div with the mouse
					(and scrolling the background) */

				retina.css({
					left				: left - sizes.retina.width/2,
					top					: top - sizes.retina.height/2,
					backgroundPosition	: '-'+(((m_nOriginalWidth - m_nMagnifierWidth)/m_nWidth)*left)+'px -'+(((m_nOriginalHeight - m_nMagnifierWidth)/m_nHeight)*top)+'px'
				});
				
			}).mouseleave(function(){
				retina.stop(true,true).fadeOut('fast');
			}).mouseenter(function(){
				retina.stop(true,true).fadeIn('fast');
			});
		}
		
		// revealing public API
		return {

			// declare public members

			onLoading : function() {
				
				context.page.event.bindOnEnter( this );		
				
				initializeParameters( metadata.parameters );		
													
				createMarkup();
				
				if ( !context.stage.isInAuthoringMode()) {
					initializeMagnifier();
				}
			},
			
			onAllComponentsReady : function () {
				initializeMagnifier();
			},
			
			onEnterPage : function() {	

			}
		};		
	};
});

